/*
   Copyright (c) 1993 Richard V. Nash.
   Copyright (c) 2000 Dan Papasian
   Copyright (C) Andrew Tridgell 2002
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "includes.h"

/*Let users know that someone is available (format the message)*/
static void getavailmess(int p, char* message)
{
 char titles[100];

 titles [0]='\0';
 AddPlayerLists(p,titles);
 sprintf (message,"%s%s Blitz (%s), Std (%s), Wild (%s), Light(%s), Bug(%s)\n"
                  "  is now available for matches.",
	player_globals.parray[p].name, titles,
	ratstrii(player_globals.parray[p].b_stats.rating, p),
        ratstrii(player_globals.parray[p].s_stats.rating, p),
	ratstrii(player_globals.parray[p].w_stats.rating, p),
	ratstrii(player_globals.parray[p].l_stats.rating, p),
        ratstrii(player_globals.parray[p].bug_stats.rating, p));
}

void getnotavailmess(int p, char* message)
{
 char titles[100];

 titles[0]='\0';
 AddPlayerLists(p,titles);
 sprintf (message,"%s%s is no longer available for matches.",
	player_globals.parray[p].name, titles);
}

void announce_avail(int p)
{
 char avail[200];
 int p1;
 if ((player_globals.parray[p].game < 0) && (CheckPFlag(p, PFLAG_OPEN))) {
   getavailmess (p,avail);

   for (p1 = 0; p1 < player_globals.p_num; p1++) {
     if (p == p1)
       continue;
     if (player_globals.parray[p1].status != PLAYER_PROMPT)
       continue;
     if (CheckPFlag(p1, PFLAG_AVAIL) && CheckPFlag(p1, PFLAG_OPEN)
         && (player_globals.parray[p1].game < 0))
       if (((player_globals.parray[p].b_stats.rating <= player_globals.parray[p1].availmax) && (player_globals.parray[p].b_stats.rating >= player_globals.parray[p1].availmin)) || (!player_globals.parray[p1].availmax))
       pprintf_prompt (p1,"\n%s\n",avail);
   }
 }
}

void announce_notavail(int p)
{
 char avail[200];
 int p1;

   getnotavailmess (p,avail);

   for (p1 = 0; p1 < player_globals.p_num; p1++) {
     if (p == p1)
       continue;
     if (player_globals.parray[p1].status != PLAYER_PROMPT)
       continue;
     if (CheckPFlag(p1, PFLAG_AVAIL) && CheckPFlag(p1, PFLAG_OPEN)
         && (player_globals.parray[p1].game < 0))
       if (((player_globals.parray[p].b_stats.rating <= player_globals.parray[p1].availmax) && (player_globals.parray[p].b_stats.rating >= player_globals.parray[p1].availmin)) || (!player_globals.parray[p1].availmax))
       pprintf_prompt (p1,"\n%s\n",avail);
   }
}

void game_ended(int g, int winner, int why)
{
  char outstr[200];
  char avail_black[200]; /* for announcing white/black avail */
  char avail_white[200];
  char avail_bugwhite[200];
  char avail_bugblack[200];
  char tmp[200];
  int p;
  int gl = game_globals.garray[g].link;
  int rate_change = 0;
  int isDraw = 0;
  int whiteResult;
  char winSymbol[10];
  char EndSymbol[10];
  char *NameOfWinner, *NameOfLoser;
  int beingplayed = 0;		/* i.e. it wasn't loaded for adjudication */
  int print_avail = 0;

  avail_white[0] = '\0';
  avail_black[0] = '\0';
  avail_bugwhite[0] = '\0';
  avail_bugblack[0] = '\0';

  beingplayed = (player_globals.parray[game_globals.garray[g].black].game == g);

  sprintf(outstr, "\n{Game %d (%s vs. %s) ", g + 1,
	  player_globals.parray[game_globals.garray[g].white].name,
	  player_globals.parray[game_globals.garray[g].black].name);
  game_globals.garray[g].result = why;
  game_globals.garray[g].winner = winner;
  if (winner == WHITE) {
    whiteResult = RESULT_WIN;
    strcpy(winSymbol, "1-0");
    NameOfWinner = player_globals.parray[game_globals.garray[g].white].name;
    NameOfLoser = player_globals.parray[game_globals.garray[g].black].name;
  } else {
    whiteResult = RESULT_LOSS;
    strcpy(winSymbol, "0-1");
    NameOfWinner = player_globals.parray[game_globals.garray[g].black].name;
    NameOfLoser = player_globals.parray[game_globals.garray[g].white].name;
  }
  switch (why) {
  case END_CHECKMATE:
    sprintf(tmp, "%s checkmated} %s\n", NameOfLoser, winSymbol);
    strcpy(EndSymbol, "Mat");
    rate_change = 1;
    break;
  case END_RESIGN:
    sprintf(tmp, "%s resigns} %s\n", NameOfLoser, winSymbol);
    strcpy(EndSymbol, "Res");
    rate_change = 1;
    break;
  case END_FLAG:
    sprintf(tmp, "%s forfeits on time} %s\n", NameOfLoser, winSymbol);
    strcpy(EndSymbol, "Fla");
    rate_change = 1;
    break;
  case END_STALEMATE:
    sprintf(tmp, "Game drawn by stalemate} 1/2-1/2\n");
    isDraw = 1;
    strcpy(EndSymbol, "Sta");
    rate_change = 1;
    whiteResult = RESULT_DRAW;
    break;
  case END_AGREEDDRAW:
    sprintf(tmp, "Game drawn by mutual agreement} 1/2-1/2\n");
    isDraw = 1;
    strcpy(EndSymbol, "Agr");
    rate_change = 1;
    whiteResult = RESULT_DRAW;
    break;
  case END_BOTHFLAG:
    sprintf(tmp, "Game drawn because both players ran out of time} 1/2-1/2\n");
    isDraw = 1;
    strcpy(EndSymbol, "Fla");
    rate_change = 1;
    whiteResult = RESULT_DRAW;
    break;
  case END_REPETITION:
    sprintf(tmp, "Game drawn by repetition} 1/2-1/2\n");
    isDraw = 1;
    strcpy(EndSymbol, "Rep");
    rate_change = 1;
    whiteResult = RESULT_DRAW;
    break;
  case END_50MOVERULE:
    sprintf(tmp, "Game drawn by the 50 move rule} 1/2-1/2\n");
    isDraw = 1;
    strcpy(EndSymbol, "50");
    rate_change = 1;
    whiteResult = RESULT_DRAW;
    break;
  case END_ADJOURN:
    if (gl >= 0) {
      sprintf(tmp, "Bughouse game aborted.} *\n");
      whiteResult = RESULT_ABORT;
    } else {
    sprintf(tmp, "Game adjourned by mutual agreement} *\n");
    game_save(g);
    }
    break;
  case END_LOSTCONNECTION:
    sprintf(tmp, "%s lost connection; game ", NameOfWinner);
    if (CheckPFlag(game_globals.garray[g].white, PFLAG_REG)
        && CheckPFlag(game_globals.garray[g].black, PFLAG_REG)
        && gl < 0) {
      sprintf(tmp, "adjourned} *\n");
      game_save(g);
    } else
      sprintf(tmp, "aborted} *\n");
    whiteResult = RESULT_ABORT;
    break;
  case END_ABORT:
    sprintf(tmp, "Game aborted by mutual agreement} *\n");
    whiteResult = RESULT_ABORT;
    break;
  case END_COURTESY:
    sprintf(tmp, "Game courtesyaborted by %s} *\n", NameOfWinner);
    whiteResult = RESULT_ABORT;
    break;
  case END_COURTESYADJOURN:
    if (gl >= 0) {
      sprintf(tmp, "Bughouse game courtesyaborted by %s.} *\n", NameOfWinner);
      whiteResult = RESULT_ABORT;
    } else {
    sprintf(tmp, "Game courtesyadjourned by %s} *\n", NameOfWinner);
    game_save(g);
    }
    break;
  case END_NOMATERIAL:
    /* Draw by insufficient material (e.g., lone K vs. lone K) */
    sprintf(tmp, "Neither player has mating material} 1/2-1/2\n");
    isDraw = 1;
    strcpy(EndSymbol, "NM ");
    rate_change = 1;
    whiteResult = RESULT_DRAW;
    break;
  case END_FLAGNOMATERIAL:
    sprintf(tmp, "%s ran out of time and %s has no material to mate} 1/2-1/2\n",
	    NameOfLoser, NameOfWinner);
    isDraw = 1;
    strcpy(EndSymbol, "TM ");
    rate_change = 1;
    whiteResult = RESULT_DRAW;
    break;
  case END_ADJWIN:
    sprintf(tmp, "%s wins by adjudication} %s\n", NameOfWinner, winSymbol);
    strcpy(EndSymbol, "Adj");
    rate_change = 1;
    break;
  case END_ADJDRAW:
    sprintf(tmp, "Game drawn by adjudication} 1/2-1/2\n");
    isDraw = 1;
    strcpy(EndSymbol, "Adj");
    rate_change = 1;
    whiteResult = RESULT_DRAW;
    break;
  case END_ADJABORT:
    sprintf(tmp, "Game aborted by adjudication} *\n");
    whiteResult = RESULT_ABORT;
    break;
  default:
    sprintf(tmp, "Hmm, the game ended and I don't know why} *\n");
    break;
  }
  strcat(outstr, tmp);

  if (game_globals.garray[g].rated && rate_change && game_globals.garray[g].type != TYPE_BUGHOUSE)
    /* Adjust ratings; bughouse gets done later. */
    rating_update(g, -1);

  if (beingplayed) {
    int printed = 0;
    int avail_printed = 0;

    pprintf_noformat(game_globals.garray[g].white, outstr);
    pprintf_noformat(game_globals.garray[g].black, outstr);
    Bell (game_globals.garray[g].white);
    Bell (game_globals.garray[g].black);

    game_globals.garray[g].link = -1;		/*IanO: avoids recursion */
    if (gl >= 0 && game_globals.garray[gl].link >= 0) {
      pprintf_noformat(game_globals.garray[gl].white, outstr);
      pprintf_noformat(game_globals.garray[gl].black, outstr);
      if (CheckPFlag(game_globals.garray[gl].white, PFLAG_OPEN)) {
        getavailmess (game_globals.garray[gl].white, avail_bugwhite);
        print_avail = 1;
      }
      if (CheckPFlag(game_globals.garray[gl].black, PFLAG_OPEN)) {
        getavailmess (game_globals.garray[gl].black, avail_bugblack);
        print_avail = 1;
      }
      if ((game_globals.garray[g].rated) && (rate_change)) {
        /* Adjust ratings */
        rating_update(g, gl);
      }
      game_ended(gl, CToggle(winner), why);
    }

    if ((player_num_active_boards(game_globals.garray[g].white) <= 1) /* not a simul or */
         && CheckPFlag(game_globals.garray[g].white, PFLAG_OPEN)) {   /* simul is over? */
      getavailmess (game_globals.garray[g].white,avail_white);
      print_avail = 1;
    } else {    /* Part of an ongoing simul!  Let's shrink the array. */
      
    }
    
    if (CheckPFlag(game_globals.garray[g].black, PFLAG_OPEN)) {
      getavailmess (game_globals.garray[g].black,avail_black);
      print_avail = 1;
    }

    for (p = 0; p < player_globals.p_num; p++) {
      if ((p == game_globals.garray[g].white) || (p == game_globals.garray[g].black))
	continue;
      if (player_globals.parray[p].status != PLAYER_PROMPT)
	continue;

      if (CheckPFlag(p, PFLAG_GIN) || player_is_observe(p, g)) {
        pprintf_noformat(p, outstr);
        printed = 1;
      }

      if (CheckPFlag(p, PFLAG_AVAIL) && (CheckPFlag(p, PFLAG_OPEN)) && (player_globals.parray[p].game < 0) && (print_avail)) {
        if (((player_globals.parray[game_globals.garray[g].white].b_stats.rating <= player_globals.parray[p].availmax) && (player_globals.parray[game_globals.garray[g].white].b_stats.rating >= player_globals.parray[p].availmin)) || (!player_globals.parray[p].availmax)) {
          pprintf (p,"\n%s",avail_white);
          avail_printed = 1;
        }
        if (((player_globals.parray[game_globals.garray[g].black].b_stats.rating <= player_globals.parray[p].availmax) && (player_globals.parray[game_globals.garray[g].black].b_stats.rating >= player_globals.parray[p].availmin)) || (!player_globals.parray[p].availmax)) {
          pprintf (p,"\n%s",avail_black);
          avail_printed = 1;
        }
        if (gl != -1) /* bughouse ? */ {
          if (((player_globals.parray[game_globals.garray[gl].white].b_stats.rating <= player_globals.parray[p].availmax) && (player_globals.parray[game_globals.garray[gl].white].b_stats.rating >= player_globals.parray[p].availmin)) || (!player_globals.parray[p].availmax)) {
            pprintf (p,"\n%s",avail_bugwhite);
            avail_printed = 1;
          }
          if (((player_globals.parray[game_globals.garray[gl].black].b_stats.rating <= player_globals.parray[p].availmax) && (player_globals.parray[game_globals.garray[gl].black].b_stats.rating >= player_globals.parray[p].availmin)) || (!player_globals.parray[p].availmax)) {
            pprintf (p,"\n%s",avail_bugblack);
            avail_printed = 1;
          }
        }
        if (avail_printed) {
          avail_printed = 0;
	  printed = 1; 
	  pprintf (p,"\n");
        }
      }

      if (printed) {
        send_prompt(p);
        printed = 0;
      }
    }

    if (!(game_globals.garray[g].rated && rate_change)) {
      pprintf(game_globals.garray[g].white, "No ratings adjustment done.\n");
      pprintf(game_globals.garray[g].black, "No ratings adjustment done.\n");
    } 
  }

  if (rate_change && gl < 0)
    game_write_complete(g, isDraw, EndSymbol);
  /* Mail off the moves */
  if (CheckPFlag(game_globals.garray[g].white, PFLAG_AUTOMAIL)) {
    pcommand(game_globals.garray[g].white, "mailmoves");
  }
  if (CheckPFlag(game_globals.garray[g].black, PFLAG_AUTOMAIL)) {
    pcommand(game_globals.garray[g].black, "mailmoves");
  }
  if (!((player_globals.parray[game_globals.garray[g].white].simul_info != NULL) &&
         (player_globals.parray[game_globals.garray[g].white].simul_info->numBoards))) {
    player_globals.parray[game_globals.garray[g].white].num_white++;
    PFlagOFF(game_globals.garray[g].white, PFLAG_LASTBLACK);
    player_globals.parray[game_globals.garray[g].black].num_black++;
    PFlagON(game_globals.garray[g].black, PFLAG_LASTBLACK);
  }
  player_globals.parray[game_globals.garray[g].white].last_opponent = 
	  strdup(game_globals.garray[g].black_name);
  player_globals.parray[game_globals.garray[g].black].last_opponent = 
	  strdup(game_globals.garray[g].white_name);
  if (beingplayed) {
    player_globals.parray[game_globals.garray[g].white].game = -1;
    player_globals.parray[game_globals.garray[g].black].game = -1;
    player_globals.parray[game_globals.garray[g].white].opponent = -1;
    player_globals.parray[game_globals.garray[g].black].opponent = -1;
    if (game_globals.garray[g].white != command_globals.commanding_player)
      send_prompt(game_globals.garray[g].white);
    if (game_globals.garray[g].black != command_globals.commanding_player)
      send_prompt(game_globals.garray[g].black);
    if ((player_globals.parray[game_globals.garray[g].white].simul_info != NULL) && 
         (player_globals.parray[game_globals.garray[g].white].simul_info->numBoards))
      player_simul_over(game_globals.garray[g].white, g, whiteResult);
  }
  game_finish(g); 
}

static int was_promoted(struct game *g, int f, int r)
{
#define BUGHOUSE_PAWN_REVERT 1
#ifdef BUGHOUSE_PAWN_REVERT
  int i;

  for (i = g->numHalfMoves-2; i > 0; i -= 2) {
    if (g->moveList[i].toFile == f && g->moveList[i].toRank == r) {
      if (g->moveList[i].piecePromotionTo)
	return 1;
      if (g->moveList[i].fromFile == ALG_DROP)
	return 0;
      f = g->moveList[i].fromFile;
      r = g->moveList[i].fromRank;
    }
  }
#endif
  return 0;
}

int pIsPlaying (int p)
{
  int g = player_globals.parray[p].game;
  int p1 = player_globals.parray[p].opponent;

  if (g < 0 || game_globals.garray[g].status != GAME_ACTIVE) {
    pprintf (p, "You are not playing a game.\n");
    return 0;
  } else if (game_globals.garray[g].white != p && game_globals.garray[g].black != p) {
    /* oh oh; big bad game bug. */
    d_printf("BUG:  Player %s playing game %d according to player_globals.parray,"
             "\n      but not according to game_globals.garray.\n", player_globals.parray[p].name, g+1);
    pprintf (p, "Disconnecting you from game number %d.\n", g+1);
    player_globals.parray[p].game = -1;
    if (p1 >= 0 && player_globals.parray[p1].game == g
        && game_globals.garray[g].white != p1 && game_globals.garray[g].black != p1) {
      pprintf (p1, "Disconnecting you from game number %d.\n", g+1);
      player_globals.parray[p1].game = -1;
    }
    return 0;
  }
  else return 1;
}

void process_move(int p, char *command)
{
  int g, result, len, i;
  struct move_t move;
  unsigned now = 0;

  if (player_globals.parray[p].game < 0) {
    pprintf(p, "You are not playing or examining a game.\n");
    return;
  }
  decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);
  g = player_globals.parray[p].game;

  if (game_globals.garray[g].status == GAME_SETUP) {
    if (!attempt_drop(p,g,command)) {
      pprintf(p, "You are still setting up the position.\n");
      pprintf(p, "Type: 'setup done' when you are finished editing.\n");
    } else
    send_board_to(g, p); 
    return;
  }

  if (game_globals.garray[g].status != GAME_EXAMINE) {
    if (!pIsPlaying(p)) return;

    if (player_globals.parray[p].side != game_globals.garray[g].game_state.onMove) {
      pprintf(p, "It is not your move.\n");
      return;
    }
    if (game_globals.garray[g].clockStopped) {
      pprintf(p, "Game clock is paused, use \"unpause\" to resume.\n");
      return;
    }
  }
  if ((len = strlen(command)) > 1) {
    if (command[len - 2] == '=') {
      switch (tolower(command[strlen(command) - 1])) {
      case 'n':
	player_globals.parray[p].promote = KNIGHT;
	break;
      case 'b':
	player_globals.parray[p].promote = BISHOP;
	break;
      case 'r':
	player_globals.parray[p].promote = ROOK;
	break;
      case 'q':
	player_globals.parray[p].promote = QUEEN;
	break;
      default:
	pprintf(p, "Don't understand that move.\n");
	return;
	break;
      }
    }
  }
  switch (parse_move(command, &game_globals.garray[g].game_state, &move, player_globals.parray[p].promote)) {
  case MOVE_ILLEGAL:
    pprintf(p, "Illegal move.\n");
    return;
    break;
  case MOVE_AMBIGUOUS:
    pprintf(p, "Ambiguous move.\n");
    return;
    break;
  default:
    break;
  }

  if (game_globals.garray[g].status == GAME_EXAMINE) {
    game_globals.garray[g].numHalfMoves++;
    if (game_globals.garray[g].numHalfMoves > game_globals.garray[g].examMoveListSize) {
      game_globals.garray[g].examMoveListSize += 20;	/* Allocate 20 moves at a time */
      game_globals.garray[g].examMoveList = (struct move_t *) realloc(game_globals.garray[g].examMoveList, sizeof(struct move_t) * game_globals.garray[g].examMoveListSize);
    }
    result = execute_move(&game_globals.garray[g].game_state, &move, 1);
    move.atTime = now;
    move.tookTime = 0;
    MakeFENpos(g, move.FENpos);
    game_globals.garray[g].examMoveList[game_globals.garray[g].numHalfMoves - 1] = move;
    /* roll back time */
    if (game_globals.garray[g].game_state.onMove == WHITE) {
      game_globals.garray[g].wTime += (game_globals.garray[g].lastDecTime - game_globals.garray[g].lastMoveTime);
    } else {
      game_globals.garray[g].bTime += (game_globals.garray[g].lastDecTime - game_globals.garray[g].lastMoveTime);
    }
    now = tenth_secs();
    if (game_globals.garray[g].numHalfMoves == 0)
      game_globals.garray[g].timeOfStart = now;
    game_globals.garray[g].lastMoveTime = now;
    game_globals.garray[g].lastDecTime = now;

  } else {			/* real game */
    i = player_globals.parray[p].opponent;
    if ((player_globals.parray[i].simul_info != NULL) && (player_globals.parray[i].simul_info->numBoards &&
	 (player_globals.parray[i].simul_info->boards[player_globals.parray[i].simul_info->onBoard] != g))) {
      pprintf(p, "It isn't your turn: wait until the simul giver is at your board.\n");
      return;
    }
    if (net_globals.con[player_globals.parray[p].socket]->timeseal) {	/* does he use timeseal? */
      if (player_globals.parray[p].side == WHITE) {
	game_globals.garray[g].wLastRealTime = game_globals.garray[g].wRealTime;
	game_globals.garray[g].wTimeWhenMoved = net_globals.con[player_globals.parray[p].socket]->time;
	if (((game_globals.garray[g].wTimeWhenMoved - game_globals.garray[g].wTimeWhenReceivedMove) < 0) ||
	    (game_globals.garray[g].wTimeWhenReceivedMove == 0)) {
	  /* might seem weird - but could be caused by a person moving BEFORE
	     he receives the board pos (this is possible due to lag) but it's
	     safe to say he moved in 0 secs :-) */
	  game_globals.garray[g].wTimeWhenReceivedMove = game_globals.garray[g].wTimeWhenMoved;
	} else {
	  game_globals.garray[g].wRealTime -= game_globals.garray[g].wTimeWhenMoved - game_globals.garray[g].wTimeWhenReceivedMove;
	}
      } else if (player_globals.parray[p].side == BLACK) {
	game_globals.garray[g].bLastRealTime = game_globals.garray[g].bRealTime;
	game_globals.garray[g].bTimeWhenMoved = net_globals.con[player_globals.parray[p].socket]->time;
	if (((game_globals.garray[g].bTimeWhenMoved - game_globals.garray[g].bTimeWhenReceivedMove) < 0) ||
	    (game_globals.garray[g].bTimeWhenReceivedMove == 0)) {
	  /* might seem weird - but could be caused by a person moving BEFORE
	     he receives the board pos (this is possible due to lag) but it's
	     safe to say he moved in 0 secs :-) */
	  game_globals.garray[g].bTimeWhenReceivedMove = game_globals.garray[g].bTimeWhenMoved;
	} else {
	  game_globals.garray[g].bRealTime -= game_globals.garray[g].bTimeWhenMoved - game_globals.garray[g].bTimeWhenReceivedMove;
	}
      }
    }
    /* we need to reset the opp's time for receiving the board since the
       timeseal decoder only alters the time if it's 0 Otherwise the time
       would be changed if the player did a refresh which would screw up
       the timings */
    if (player_globals.parray[p].side == WHITE) {
      game_globals.garray[g].bTimeWhenReceivedMove = 0;
    } else {
      game_globals.garray[g].wTimeWhenReceivedMove = 0;
    }

    game_update_time(g);

    /* maybe add autoflag here in the future? */
    if (net_globals.con[player_globals.parray[p].socket]->timeseal) {	/* does he use timeseal? */
      if (player_globals.parray[p].side == WHITE) {
	game_globals.garray[g].wRealTime += game_globals.garray[g].wIncrement * 100;
	game_globals.garray[g].wTime = game_globals.garray[g].wRealTime / 100;	/* remember to conv to
							   tenth secs */
      } else if (player_globals.parray[p].side == BLACK) {
	game_globals.garray[g].bRealTime += game_globals.garray[g].bIncrement * 100;	/* conv to ms */
	game_globals.garray[g].bTime = game_globals.garray[g].bRealTime / 100;	/* remember to conv to
							   tenth secs */
      }
    } else {
      if (game_globals.garray[g].game_state.onMove == BLACK) {
	game_globals.garray[g].bTime += game_globals.garray[g].bIncrement;
      }
      if (game_globals.garray[g].game_state.onMove == WHITE) {
	game_globals.garray[g].wTime += game_globals.garray[g].wIncrement;
      }
    }

    /* Do the move */
    game_globals.garray[g].numHalfMoves++;
    if (game_globals.garray[g].numHalfMoves > game_globals.garray[g].moveListSize) {
      game_globals.garray[g].moveListSize += 20;	/* Allocate 20 moves at a time */
      game_globals.garray[g].moveList = (struct move_t *) realloc(game_globals.garray[g].moveList, sizeof(struct move_t) * game_globals.garray[g].moveListSize);
    }
    result = execute_move(&game_globals.garray[g].game_state, &move, 1);
    if (result == MOVE_OK && game_globals.garray[g].link >= 0 && move.pieceCaptured != NOPIECE) {
      /* transfer captured piece to partner */
      /* check if piece reverts to a pawn */
      if (was_promoted(&game_globals.garray[g], move.toFile, move.toRank))
        update_holding(game_globals.garray[g].link, colorval(move.pieceCaptured) | PAWN);
      else
        update_holding(game_globals.garray[g].link, move.pieceCaptured);
    }
    now = tenth_secs();
    move.atTime = now;
    if (game_globals.garray[g].numHalfMoves > 1) {
      move.tookTime = move.atTime - game_globals.garray[g].lastMoveTime;
    } else {
      move.tookTime = move.atTime - game_globals.garray[g].startTime;
    }
    game_globals.garray[g].lastMoveTime = now;
    game_globals.garray[g].lastDecTime = now;
    move.wTime = game_globals.garray[g].wTime;
    move.bTime = game_globals.garray[g].bTime;

    if (net_globals.con[player_globals.parray[p].socket]->timeseal) {	/* does he use timeseal? */
      if (player_globals.parray[p].side == WHITE) {
	move.tookTime = (game_globals.garray[player_globals.parray[p].game].wTimeWhenMoved -
			 game_globals.garray[player_globals.parray[p].game].wTimeWhenReceivedMove) / 100;
      } else {
	move.tookTime = (game_globals.garray[player_globals.parray[p].game].bTimeWhenMoved -
			 game_globals.garray[player_globals.parray[p].game].bTimeWhenReceivedMove) / 100;
      }
    }

    MakeFENpos(g, move.FENpos);
    game_globals.garray[g].moveList[game_globals.garray[g].numHalfMoves - 1] = move;
  }

  send_boards(g);

  if (result == MOVE_ILLEGAL) {
    pprintf(p, "Internal error, illegal move accepted!\n");
  }
  if ((result == MOVE_OK) && (game_globals.garray[g].status == GAME_EXAMINE)) {
    int p1;

    for (p1 = 0; p1 < player_globals.p_num; p1++) {
      if (player_globals.parray[p1].status != PLAYER_PROMPT)
	continue;
      if (player_is_observe(p1, g) || player_globals.parray[p1].game == g) {
	pprintf(p1, "%s moves: %s\n", player_globals.parray[p].name, move.algString);
      }
    }
  }
  if (result == MOVE_CHECKMATE) {
    if (game_globals.garray[g].status == GAME_EXAMINE) {
      int p1;

      for (p1 = 0; p1 < player_globals.p_num; p1++) {
	if (player_globals.parray[p1].status != PLAYER_PROMPT)
	  continue;
	if (player_is_observe(p1, g) || player_globals.parray[p1].game == g) {
	  pprintf(p1, "%s has been checkmated.\n",
		  (CToggle(game_globals.garray[g].game_state.onMove) == BLACK) ? "White" : "Black");
	}
      }
    } else {
      game_ended(g, CToggle(game_globals.garray[g].game_state.onMove), END_CHECKMATE);
    }
  }
  if (result == MOVE_STALEMATE) {
    if (game_globals.garray[g].status == GAME_EXAMINE) {
      int p1;

      for (p1 = 0; p1 < player_globals.p_num; p1++) {
	if (player_globals.parray[p1].status != PLAYER_PROMPT)
	  continue;
	if (player_is_observe(p1, g) || player_globals.parray[p1].game == g) {
	  pprintf(p1, "Stalemate.\n");
	}
      }
    } else {
      game_ended(g, CToggle(game_globals.garray[g].game_state.onMove), END_STALEMATE);
    }
  }
  if (result == MOVE_NOMATERIAL) {
    if (game_globals.garray[g].status == GAME_EXAMINE) {
      int p1;

      for (p1 = 0; p1 < player_globals.p_num; p1++) {
	if (player_globals.parray[p1].status != PLAYER_PROMPT)
	  continue;
	if (player_is_observe(p1, g) || player_globals.parray[p1].game == g) {
	  pprintf(p1, "No mating material.\n");
	}
      }
    } else {
      game_ended(g, CToggle(game_globals.garray[g].game_state.onMove), END_NOMATERIAL);
    }
  }
}

int com_resign(int p, param_list param)
{
  int g, o, oconnected;

  if (param[0].type == TYPE_NULL) {
    g = player_globals.parray[p].game;
    if (!pIsPlaying(p))
      return COM_OK;
    else {
      decline_withdraw_offers(p, -1, -1, DO_DECLINE);
      game_ended(g, (game_globals.garray[g].white == p) ? BLACK : WHITE, END_RESIGN);
    }
  } else if (FindPlayer(p, param[0].val.word, &o, &oconnected)) {
    g = game_new();
    if (game_read(g, p, o) < 0) {
      if (game_read(g, o, p) < 0) {
	pprintf(p, "You have no stored game with %s\n", player_globals.parray[o].name);
        if (!oconnected)
          player_remove(o);
        return COM_OK;
      } else {
	game_globals.garray[g].white = o;
	game_globals.garray[g].black = p;
      }
    } else {
      game_globals.garray[g].white = p;
      game_globals.garray[g].black = o;
    }
    pprintf(p, "You resign your stored game with %s\n", player_globals.parray[o].name);
    pcommand(p, "message %s I have resigned our stored game \"%s vs. %s.\"",
	     player_globals.parray[o].name,
	     player_globals.parray[game_globals.garray[g].white].name,
	     player_globals.parray[game_globals.garray[g].black].name);
    game_delete(game_globals.garray[g].white, game_globals.garray[g].black);
    game_ended(g, (game_globals.garray[g].white == p) ? BLACK : WHITE, END_RESIGN);
    if (!oconnected)
      player_remove(o);
  }
  return COM_OK;
}

static int Check50MoveRule (int p, int g)
{
  int num_reversible = game_globals.garray[g].numHalfMoves;

  if (game_globals.garray[g].game_state.lastIrreversable >= 0) {
    num_reversible -= game_globals.garray[g].game_state.lastIrreversable;
  }
  if (num_reversible > 99) {
    game_ended(g, (game_globals.garray[g].white == p) ? BLACK : WHITE, END_50MOVERULE);
    return 1;
  }
  return 0;
}

static char *GetFENpos (int g, int half_move)
{
  if (half_move < 0)
    return game_globals.garray[g].FENstartPos;
  else return game_globals.garray[g].moveList[half_move].FENpos;
}

static int CheckRepetition (int p, int g)
{
  struct pending* pend;
  int move_num;
  int flag1 = 1, flag2 = 1;
  char *pos1 = GetFENpos (g, game_globals.garray[g].numHalfMoves - 1);
  char *pos2 = GetFENpos (g, game_globals.garray[g].numHalfMoves);
  char *pos;

  if (game_globals.garray[g].numHalfMoves < 8)  /* can't have three repeats any quicker. */
    return 0;

  for (move_num = game_globals.garray[g].game_state.lastIrreversable;
       move_num < game_globals.garray[g].numHalfMoves - 1; move_num++) {
    pos = GetFENpos (g, move_num);
    if (strlen(pos1) == strlen(pos) && !strcmp(pos1, pos))
      flag1++;
    if (strlen(pos2) == strlen(pos) && !strcmp(pos2, pos))
      flag2++;
  }
  if (flag1 >= 3 || flag2 >= 3) {
    if ((pend = find_pend(player_globals.parray[p].opponent, p, PEND_DRAW)) != NULL) {
      delete_pending(pend);
      decline_withdraw_offers(p, -1, -1,DO_DECLINE);
    }
    game_ended(g, (game_globals.garray[g].white == p) ? BLACK : WHITE, END_REPETITION);
    return 1;
  }
  else return 0;
}

int com_draw(int p, param_list param)
{
  struct pending* pend;
  int p1, g = player_globals.parray[p].game;

  if (!pIsPlaying(p)) {
    return COM_OK;
  }
  if (Check50MoveRule (p, g) || CheckRepetition(p, g)) {
    return COM_OK;
  }
  p1 = player_globals.parray[p].opponent;

  if ((player_globals.parray[p1].simul_info != NULL) && (player_globals.parray[p1].simul_info->numBoards &&
        player_globals.parray[p1].simul_info->boards[player_globals.parray[p1].simul_info->onBoard] != g)) {
    pprintf(p, "You can only make requests when the simul player is at your board.\n");
    return COM_OK;
  }

  if ((pend = (find_pend(player_globals.parray[p].opponent, p, PEND_DRAW))) != NULL) {
    delete_pending(pend);
    decline_withdraw_offers(p, -1, -1,DO_DECLINE);
    game_ended(g, (game_globals.garray[g].white == p) ? BLACK : WHITE, END_AGREEDDRAW);
  } else {
    pprintf(player_globals.parray[p].opponent, "\n");
    pprintf_highlight(player_globals.parray[p].opponent, "%s", player_globals.parray[p].name);
    pprintf_prompt(player_globals.parray[p].opponent, " offers you a draw.\n");
    pprintf(p, "Draw request sent.\n");
    add_request(p, player_globals.parray[p].opponent, PEND_DRAW);
  }
  return COM_OK;
}

int com_pause(int p, param_list param)
{
  int g, now;
  struct pending* pend;

  if (!pIsPlaying(p)) {
    return COM_OK;
  }
  g = player_globals.parray[p].game;
  if (game_globals.garray[g].wTime == 0) {
    pprintf(p, "You can't pause untimed games.\n");
    return COM_OK;
  }
  if (game_globals.garray[g].clockStopped) {
    pprintf(p, "Game is already paused, use \"unpause\" to resume.\n");
    return COM_OK;
  }
  if ((pend = find_pend(player_globals.parray[p].opponent, p, PEND_PAUSE)) != NULL) {
    delete_pending(pend);
    game_globals.garray[g].clockStopped = 1;
    /* Roll back the time */
    if (game_globals.garray[g].game_state.onMove == WHITE) {
      game_globals.garray[g].wTime += (game_globals.garray[g].lastDecTime - game_globals.garray[g].lastMoveTime);
    } else {
      game_globals.garray[g].bTime += (game_globals.garray[g].lastDecTime - game_globals.garray[g].lastMoveTime);
    }
    now = tenth_secs();
    if (game_globals.garray[g].numHalfMoves == 0)
      game_globals.garray[g].timeOfStart = now;
    game_globals.garray[g].lastMoveTime = now;
    game_globals.garray[g].lastDecTime = now;
    send_boards(g);
    pprintf_prompt(player_globals.parray[p].opponent, "\n%s accepted pause. Game clock paused.\n",
		   player_globals.parray[p].name);
    pprintf(p, "Game clock paused.\n");
  } else {
    pprintf(player_globals.parray[p].opponent, "\n");
    pprintf_highlight(player_globals.parray[p].opponent, "%s", player_globals.parray[p].name);
    pprintf_prompt(player_globals.parray[p].opponent, " requests to pause the game.\n");
    pprintf(p, "Pause request sent.\n");
    add_request(p, player_globals.parray[p].opponent, PEND_PAUSE);
  }
  return COM_OK;
}

int com_unpause(int p, param_list param)
{
  int g;
  int now;
  struct pending* pend;

  if (!pIsPlaying(p)) {
    return COM_OK;
  }

  g = player_globals.parray[p].game;

  if (!game_globals.garray[g].clockStopped) {
    pprintf(p, "Game is not paused.\n");
    return COM_OK;
  }
  if ((pend = find_pend(player_globals.parray[p].opponent, p, PEND_UNPAUSE)) != NULL) {
    delete_pending(pend);
    game_globals.garray[g].clockStopped = 0;
    now = tenth_secs();
    if (game_globals.garray[g].numHalfMoves == 0)
      game_globals.garray[g].timeOfStart = now;
    game_globals.garray[g].lastMoveTime = now;
    game_globals.garray[g].lastDecTime = now;
    send_boards(g);
    pprintf(p, "Game clock resumed.\n");
    pprintf_prompt(player_globals.parray[p].opponent, "\nGame clock resumed.\n");
  } else {
    pprintf(player_globals.parray[p].opponent, "\n");
    pprintf_highlight(player_globals.parray[p].opponent, "%s", player_globals.parray[p].name);
    pprintf_prompt(player_globals.parray[p].opponent, " requests to unpause the game.\n");
    pprintf(p, "Unpause request sent.\n");
    add_request(p, player_globals.parray[p].opponent, PEND_UNPAUSE);
  }
  return COM_OK;
}

int com_abort(int p, param_list param)
{
  struct pending* pend;
  int p1, g, myColor, yourColor, myGTime, yourGTime;
  int courtesyOK = 1;

  g = player_globals.parray[p].game;
  if (!pIsPlaying(p))
    return COM_OK;

  p1 = player_globals.parray[p].opponent;
  if (p == game_globals.garray[g].white) {
    myColor = WHITE;
    yourColor = BLACK;
    myGTime = game_globals.garray[g].wTime;
    yourGTime = game_globals.garray[g].bTime;
  } else {
    myColor = BLACK;
    yourColor = WHITE;
    myGTime = game_globals.garray[g].bTime;
    yourGTime = game_globals.garray[g].wTime;
  }
  if ((player_globals.parray[p1].simul_info != NULL) && 
     (player_globals.parray[p1].simul_info->numBoards &&
        player_globals.parray[p1].simul_info->boards[player_globals.parray[p1].simul_info->onBoard] != g)) {
    pprintf(p, "You can only make requests when the simul player is at your board.\n");
    return COM_OK;
  }
  if ((pend = find_pend(p1, p, PEND_ABORT)) != NULL) {
    delete_pending(pend);
    decline_withdraw_offers(p, -1, -1,DO_DECLINE);
    game_ended(g, yourColor, END_ABORT);
  } else {
    game_update_time(g);

    if (net_globals.con[player_globals.parray[p].socket]->timeseal
        && game_globals.garray[g].game_state.onMove == myColor
        && game_globals.garray[g].flag_pending == FLAG_ABORT) {
      /* It's my move, opponent has asked for abort; I lagged out,
         my timeseal prevented courtesyabort, and I am sending an abort
         request before acknowledging (and processing) my opponent's
         courtesyabort.  OK, let's abort already :-). */
      decline_withdraw_offers(p, -1, -1,DO_DECLINE);
      game_ended(g, yourColor, END_ABORT);
    }

    if (net_globals.con[player_globals.parray[p1].socket]->timeseal) {	/* opp uses timeseal? */

      int yourRealTime = (myColor == WHITE  ?  game_globals.garray[g].bRealTime
       	                                    :  game_globals.garray[g].wRealTime);
      if (myGTime > 0 && yourGTime <= 0 && yourRealTime > 0) {
        /* Override courtesyabort; opponent still has time.  Check for lag. */
        courtesyOK = 0;

        if (game_globals.garray[g].game_state.onMove != myColor
            && game_globals.garray[g].flag_pending != FLAG_CHECKING) {
          /* Opponent may be lagging; let's ask. */
          game_globals.garray[g].flag_pending = FLAG_ABORT;
          game_globals.garray[g].flag_check_time = time(0);
          pprintf(p, "Opponent has timeseal; trying to courtesyabort.\n");
          pprintf(p1, "\n[G]\n");
          return COM_OK;
        }
      }
    }

    if (myGTime > 0 && yourGTime <= 0 && courtesyOK) {
      /* player wants to abort + opponent is out of time = courtesyabort */
      pprintf(p, "Since you have time, and your opponent has none, the game has been aborted.");
      pprintf(p1, "Your opponent has aborted the game rather than calling your flag.");
      decline_withdraw_offers(p, -1, -1, DO_DECLINE);
      game_ended(g, myColor, END_COURTESY);
    } else {
      pprintf(p1, "\n");
      pprintf_highlight(p1, "%s", player_globals.parray[p].name);
      pprintf(p1, " would like to abort the game; ");
      pprintf_prompt(p1, "type \"abort\" to accept.\n");
      pprintf(p, "Abort request sent.\n");
      add_request(p, p1, PEND_ABORT);
    }
  }
  return COM_OK;
}

static int player_has_mating_material(struct game_state_t *gs, int color)
{
  int i, j;
  int piece;
  int minor_pieces = 0;

  for (i = 0; i < 8; i++)
    for (j = 0; j < 8; j++) {
      piece = gs->board[i][j];
      switch (piecetype(piece)) {
      case BISHOP:
      case KNIGHT:
	if (iscolor(piece, color))
	  minor_pieces++;
	break;
      case KING:
      case NOPIECE:
	break;
      default:
	if (iscolor(piece, color))
	  return 1;
      }
    }
  return ((minor_pieces > 1) ? 1 : 0);
}

int com_flag(int p, param_list param)
{
  int g;
  int myColor;

  if (!pIsPlaying(p)) {
    return COM_OK;
  }
  g = player_globals.parray[p].game;
  myColor = (p == game_globals.garray[g].white ? WHITE : BLACK);
  if (game_globals.garray[g].type == TYPE_UNTIMED) {
    pprintf(p, "You can't flag an untimed game.\n");
    return COM_OK;
  }
  if (game_globals.garray[g].numHalfMoves < 2) {
    pprintf(p, "You cannot flag before both players have moved.\nUse abort instead.\n");
    return COM_OK;
  }
  game_update_time(g);

  {
    int myTime, yourTime, opp = player_globals.parray[p].opponent, serverTime;

    if (net_globals.con[player_globals.parray[p].socket]->timeseal) {    /* does caller use timeseal? */
      myTime = (myColor == WHITE  ?  game_globals.garray[g].wRealTime
                                  :  game_globals.garray[g].bRealTime);
    } else {
      myTime = (myColor == WHITE  ?  game_globals.garray[g].wTime  :  game_globals.garray[g].bTime);
    }
    serverTime = (myColor == WHITE  ?  game_globals.garray[g].bTime  :  game_globals.garray[g].wTime);

    if (net_globals.con[player_globals.parray[opp].socket]->timeseal) {	/* opp uses timeseal? */
      yourTime = (myColor == WHITE  ?  game_globals.garray[g].bRealTime
                                    :  game_globals.garray[g].wRealTime);
    } else {
      yourTime = serverTime;
    }

    /* the clocks to compare are now in myTime and yourTime */

    if ((myTime <= 0) && (yourTime <= 0)) {
      decline_withdraw_offers(p, -1, -1,DO_DECLINE);
      game_ended(g, myColor, END_BOTHFLAG);
      return COM_OK;
    }
    if (yourTime > 0) {
      /* Opponent still has time, but if that's only because s/he
       * may be lagging, we should ask for an acknowledgement and then
       * try to call the flag. */

      if (serverTime <= 0 && game_globals.garray[g].game_state.onMove != myColor
          && game_globals.garray[g].flag_pending != FLAG_CHECKING) {

        /* server time thinks opponent is down, but RealTIme disagrees.
         * ask client to acknowledge it's alive. */

        game_globals.garray[g].flag_pending = FLAG_CALLED;
        game_globals.garray[g].flag_check_time = time(0);
        pprintf(p, "Opponent has timeseal; checking if (s)he's lagging.\n");
        pprintf (opp, "\n[G]\n");
        return COM_OK;
      }

      /* if we're here, it means one of:
       * 1. the server agrees opponent has time, whether lagging or not.
       * 2. opp. has timeseal (if yourTime != serverTime), had time left
       *    after the last move (yourTime > 0), and it's still your move.
       * 3. we're currently checking a flag call after having receiving
       *    acknowledgement from the other timeseal (and would have reset
       *    yourTime if the flag were down). */

      pprintf(p, "Your opponent is not out of time!\n");
      return COM_OK;
    }
  }

  decline_withdraw_offers(p, -1, -1,DO_DECLINE);
  if (player_has_mating_material(&game_globals.garray[g].game_state, myColor))
    game_ended(g, myColor, END_FLAG);
  else
    game_ended(g, myColor, END_FLAGNOMATERIAL);
  return COM_OK;
}

int com_adjourn(int p, param_list param)
{
  struct pending* pend;
  int p1, g, myColor, yourColor;

  if (!pIsPlaying(p))
    return COM_OK;

  p1 = player_globals.parray[p].opponent;
  g = player_globals.parray[p].game;
  if (!CheckPFlag(p, PFLAG_REG) || !CheckPFlag(p, PFLAG_REG)) {
    pprintf(p, "Both players must be registered to adjourn a game.  Use \"abort\".\n");
    return COM_OK;
  }
  if (game_globals.garray[g].link >= 0) {
    pprintf(p, "Bughouse games cannot be adjourned.\n");
    return COM_OK;
  }
  myColor = (p == game_globals.garray[g].white ? WHITE : BLACK);
  yourColor = (myColor == WHITE ? BLACK : WHITE);

  if ((pend = find_pend(p1, p, PEND_ADJOURN)) != NULL) {
    delete_pending(pend);
    decline_withdraw_offers(p, -1, -1,DO_DECLINE);
    game_ended(player_globals.parray[p].game, yourColor, END_ADJOURN);
  } else {
    game_update_time(g);
    if (((myColor == WHITE) && (game_globals.garray[g].wTime > 0) && (game_globals.garray[g].bTime <= 0))
	|| ((myColor == BLACK) && (game_globals.garray[g].bTime > 0) && (game_globals.garray[g].wTime <= 0))) {
/* player wants to adjourn + opponent is out of time = courtesyadjourn */
      pprintf(p, "Since you have time, and your opponent has none, the game has been adjourned.");
      pprintf(p1, "Your opponent has adjourned the game rather than calling your flag.");
      decline_withdraw_offers(p, -1, -1,DO_DECLINE);
      game_ended(g, myColor, END_COURTESYADJOURN);
    } else {
      pprintf(p1, "\n");
      pprintf_highlight(p1, "%s", player_globals.parray[p].name);
      pprintf(p1, " would like to adjourn the game; ");
      pprintf_prompt(p1, "type \"adjourn\" to accept.\n");
      pprintf(p, "Adjourn request sent.\n");
      add_request(p, p1, PEND_ADJOURN);
    }
  }
  return COM_OK;
}

int com_takeback(int p, param_list param)
{
  int nHalfMoves = 1, g, i, p1, pend_half_moves;
  struct pending* from;

  if (!pIsPlaying(p))
    return COM_OK;

  p1 = player_globals.parray[p].opponent;
  if ((player_globals.parray[p1].simul_info != NULL) && 
     (player_globals.parray[p1].simul_info->numBoards &&
        player_globals.parray[p1].simul_info->boards[player_globals.parray[p1].simul_info->onBoard] !=
        player_globals.parray[p].game)) {
    pprintf(p, "You can only make requests when the simul player is at your board.\n");
    return COM_OK;
  }

  g = player_globals.parray[p].game;
  if (game_globals.garray[g].link >= 0) {
    pprintf(p, "Takeback not implemented for bughouse games yet.\n");
    return COM_OK;
  }
  if (param[0].type == TYPE_INT) {
    nHalfMoves = param[0].val.integer;
    if (nHalfMoves <= 0) {
      pprintf (p,"You can't takeback less than 1 move.\n");
      return COM_OK;
    }
  }
  if ((from = find_pend(player_globals.parray[p].opponent, p, PEND_TAKEBACK)) != NULL) {
    pend_half_moves = from->wtime;
    delete_pending(from);
    if (pend_half_moves == nHalfMoves) {
      /* Doing the takeback */
      decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);
      for (i = 0; i < nHalfMoves; i++) {
	if (backup_move(g, REL_GAME) != MOVE_OK) {
	  pprintf(game_globals.garray[g].white, "Can only backup %d moves\n", i);
	  pprintf(game_globals.garray[g].black, "Can only backup %d moves\n", i);
	  break;
	}
      }

      game_globals.garray[g].wTimeWhenReceivedMove = 0;
      game_globals.garray[g].bTimeWhenReceivedMove = 0;

      send_boards(g);
    } else {

      if (!game_globals.garray[g].numHalfMoves) {
        pprintf(p, "There are no moves in your game.\n");
        pprintf_prompt(player_globals.parray[p].opponent, "\n%s has declined the takeback request.\n", 
		       player_globals.parray[p].name);
        return COM_OK;
      }
 
      if (game_globals.garray[g].numHalfMoves < nHalfMoves) {
	pprintf(p, "There are only %d half moves in your game.\n", game_globals.garray[g].numHalfMoves);
	pprintf_prompt(player_globals.parray[p].opponent, "\n%s has declined the takeback request.\n", 
		       player_globals.parray[p].name);
	return COM_OK;
      }
      pprintf(p, "You disagree on the number of half-moves to takeback.\n");
      pprintf(p, "Alternate takeback request sent.\n");
      pprintf_prompt(player_globals.parray[p].opponent, "\n%s proposes a different number (%d) of half-move(s).\n", player_globals.parray[p].name, nHalfMoves);
      from = add_request(p, player_globals.parray[p].opponent, PEND_TAKEBACK);
      from->wtime = nHalfMoves;
    }
  } else {

    if (!game_globals.garray[g].numHalfMoves) {
      pprintf(p, "There are no moves in your game.\n");
      return COM_OK;
    }
    if (game_globals.garray[g].numHalfMoves < nHalfMoves) {
      pprintf(p, "There are only %d half moves in your game.\n", game_globals.garray[g].numHalfMoves);
      return COM_OK;
    }
    pprintf(player_globals.parray[p].opponent, "\n");
    pprintf_highlight(player_globals.parray[p].opponent, "%s", player_globals.parray[p].name);
    pprintf_prompt(player_globals.parray[p].opponent, " would like to take back %d half move(s).\n",
 	   nHalfMoves);
    pprintf(p, "Takeback request sent.\n");
    from = add_request(p, player_globals.parray[p].opponent, PEND_TAKEBACK);
    from->wtime = nHalfMoves;
  }
  return COM_OK;
}


int com_switch(int p, param_list param)
{
  int g = player_globals.parray[p].game, tmp, now, p1;
  char *strTmp;
  struct pending* pend;

  if (!pIsPlaying(p))
    return COM_OK;

  p1 = player_globals.parray[p].opponent;
  if ((player_globals.parray[p1].simul_info != NULL) && (player_globals.parray[p1].simul_info->numBoards &&
        player_globals.parray[p1].simul_info->boards[player_globals.parray[p1].simul_info->onBoard] != g)) {
    pprintf(p, "You can only make requests when the simul player is at your board.\n");
    return COM_OK;
  }

  if (game_globals.garray[g].link >= 0) {
    pprintf(p, "Switch not implemented for bughouse games.\n");
    return COM_OK;
  }
  if ((pend = find_pend(player_globals.parray[p].opponent, p, PEND_SWITCH)) != NULL) {
    delete_pending(pend);
    /* Doing the switch */
    decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);

    tmp = game_globals.garray[g].white;
    game_globals.garray[g].white = game_globals.garray[g].black;
    game_globals.garray[g].black = tmp;
    player_globals.parray[p].side = (player_globals.parray[p].side == WHITE) ? BLACK : WHITE;
    strTmp = strdup(game_globals.garray[g].white_name);
    strcpy(game_globals.garray[g].white_name, game_globals.garray[g].black_name);
    strcpy(game_globals.garray[g].black_name, strTmp);
    free(strTmp);

    player_globals.parray[player_globals.parray[p].opponent].side =
      (player_globals.parray[player_globals.parray[p].opponent].side == WHITE) ? BLACK : WHITE;
    /* Roll back the time */
    if (game_globals.garray[g].game_state.onMove == WHITE) {
      game_globals.garray[g].wTime += (game_globals.garray[g].lastDecTime - game_globals.garray[g].lastMoveTime);
    } else {
      game_globals.garray[g].bTime += (game_globals.garray[g].lastDecTime - game_globals.garray[g].lastMoveTime);
    }
    now = tenth_secs();
    if (game_globals.garray[g].numHalfMoves == 0)
      game_globals.garray[g].timeOfStart = now;
    game_globals.garray[g].lastMoveTime = now;
    game_globals.garray[g].lastDecTime = now;
    send_boards(g);
    return COM_OK;
  }
  if (game_globals.garray[g].rated && game_globals.garray[g].numHalfMoves > 0) {
    pprintf(p, "You cannot switch sides once a rated game is underway.\n");
    return COM_OK;
  }
  pprintf(player_globals.parray[p].opponent, "\n");
  pprintf_highlight(player_globals.parray[p].opponent, "%s", player_globals.parray[p].name);
  pprintf_prompt(player_globals.parray[p].opponent, " would like to switch sides.\nType \"accept\" to switch sides, or \"decline\" to refuse.\n");
  pprintf(p, "Switch request sent.\n");
  add_request(p, player_globals.parray[p].opponent, PEND_SWITCH);
  return COM_OK;
}

int com_time(int p, param_list param)
{
  int p1, g;

  if (param[0].type == TYPE_NULL) {
    g = player_globals.parray[p].game;
    if (!pIsPlaying(p))
      return COM_OK;
  } else {
    g = GameNumFromParam(p, &p1, &param[0]);
    if (g < 0)
      return COM_OK;
  }
  if ((g < 0) || (g >= game_globals.g_num) || (game_globals.garray[g].status != GAME_ACTIVE)) {
    pprintf(p, "There is no such game.\n");
    return COM_OK;
  }
  game_update_time(g);
  pprintf(p, "White (%s) : %d mins, %d secs\n",
	  player_globals.parray[game_globals.garray[g].white].name,
	  game_globals.garray[g].wTime / 600,
	  (game_globals.garray[g].wTime - ((game_globals.garray[g].wTime / 600) * 600)) / 10);
  pprintf(p, "Black (%s) : %d mins, %d secs\n",
	  player_globals.parray[game_globals.garray[g].black].name,
	  game_globals.garray[g].bTime / 600,
	  (game_globals.garray[g].bTime - ((game_globals.garray[g].bTime / 600) * 600)) / 10);
  return COM_OK;
}

int com_ptime(int p, param_list param)
{
  int retval, part = player_globals.parray[p].partner;

  if (part < 0) {
    pprintf(p, "You do not have a partner.\n");
    return COM_OK;
  }
  retval = pcommand (p, "time %s", player_globals.parray[part].name);
  if (retval == COM_OK)
    return COM_OK_NOPROMPT;
  else
    return retval;
}

int com_boards(int p, param_list param)
{
  char *category = NULL;
  char dname[MAX_FILENAME_SIZE];
  DIR *dirp;
  struct dirent *dp;

  if (param[0].type == TYPE_WORD)
    category = param[0].val.word;
  if (category) {
    pprintf(p, "Boards Available For Category %s:\n", category);
    sprintf(dname, "%s/%s", BOARD_DIR, category);
  } else {
    pprintf(p, "Categories Available:\n");
    sprintf(dname, "%s", BOARD_DIR);
  }
  dirp = opendir(dname);
  if (!dirp) {
    pprintf(p, "No such category %s, try \"boards\".\n", category);
    return COM_OK;
  }

/* YUK! what a mess, how about printing an ordered directory? - DAV*/

  for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
    if (!strcmp(dp->d_name, "."))
      continue;
    if (!strcmp(dp->d_name, ".."))
      continue;
    pprintf(p, "%s\n", dp->d_name);
  }
  closedir(dirp);
  return COM_OK;
}

int com_simmatch(int p, param_list param)
{
  int p1, g, adjourned;
  int num;
  char tmp[100];
  struct pending* pend;
  char* board = NULL;
  char* category = NULL;
  char fname[MAX_FILENAME_SIZE];

  if (player_globals.parray[p].game >=0) {
    if (game_globals.garray[player_globals.parray[p].game].status == GAME_EXAMINE) {
      pprintf(p, "You are still examining a game.\n");
      return COM_OK;
    }
    if (game_globals.garray[player_globals.parray[p].game].status == GAME_SETUP) {
      pprintf(p, "You are still setting up a position.\n");
      return COM_OK;
    }
  } 
  p1 = player_find_part_login(param[0].val.word);
  if (p1 < 0) {
    pprintf(p, "No user named \"%s\" is logged in.\n", param[0].val.word);
    return COM_OK;
  }
  if (p == p1) {
    pprintf(p, "You can't simmatch yourself!\n");
    return COM_OK;
  }
  if ((pend = find_pend(p1, p, PEND_SIMUL)) != NULL) {

    /* Accepting Simul ! */

    if ((player_globals.parray[p].simul_info != NULL) && 
       (player_globals.parray[p].simul_info->numBoards >= MAX_SIMUL)) {
      pprintf(p, "You are already playing the maximum of %d boards.\n", MAX_SIMUL);
      pprintf(p1, "Simul request removed, boards filled.\n");
      delete_pending(pend);
      return COM_OK;
    }
    unobserveAll(p);		/* stop observing when match starts */
    unobserveAll(p1);

    g = game_new();
    adjourned = 0;
    if (game_read(g, p, p1) >= 0) {
      adjourned = 1;
      delete_pending(pend);
    }

    if (!adjourned) {		/* no adjourned game, so begin a new game */

      if ((pend->category != NULL) && (pend->board_type != NULL)) {
        board = strdup(pend->category);
        category = strdup(pend->board_type);
      } 

      delete_pending(pend);

      if (create_new_match(g,p, p1, 0, 0, 0, 0, 0, ((board == NULL) ? "\0" : board), ((category == NULL) ? "\0" : category), 1,1) == COM_FAILED) {
	pprintf(p, "There was a problem creating the new match.\n");
	pprintf_prompt(p1, "There was a problem creating the new match.\n");
        game_remove(g);

        if (board != NULL) {
          free (board);
          free (category);
        }
	return COM_OK;
      }

      if (board != NULL) {
        free (board);
        free (category);
      }

    } else {			/* resume adjourned game */
      game_delete(p, p1);

      sprintf(tmp, "{Game %d (%s vs. %s) Continuing %s %s simul.}\n", g + 1, player_globals.parray[p].name, player_globals.parray[p1].name, rstr[game_globals.garray[g].rated], bstr[game_globals.garray[g].type]);
      pprintf(p, tmp);
      pprintf(p1, tmp);

      game_globals.garray[g].white = p;
      game_globals.garray[g].black = p1;
      game_globals.garray[g].status = GAME_ACTIVE;
      game_globals.garray[g].startTime = tenth_secs();
      game_globals.garray[g].lastMoveTime = game_globals.garray[g].startTime;
      game_globals.garray[g].lastDecTime = game_globals.garray[g].startTime;
      player_globals.parray[p].game = g;
      player_globals.parray[p].opponent = p1;
      player_globals.parray[p].side = WHITE;
      player_globals.parray[p1].game = g;
      player_globals.parray[p1].opponent = p;
      player_globals.parray[p1].side = BLACK;
      send_boards(g);
    }

    if (player_globals.parray[p].simul_info == NULL) {
      player_globals.parray[p].simul_info = (struct simul_info_t *) malloc(sizeof(struct simul_info_t));
      player_globals.parray[p].simul_info->numBoards = 0;
      player_globals.parray[p].simul_info->onBoard = 0;
      player_globals.parray[p].simul_info->num_wins = player_globals.parray[p].simul_info->num_draws
        = player_globals.parray[p].simul_info->num_losses = 0;
    }
    num = player_globals.parray[p].simul_info->numBoards;
    /*    player_globals.parray[p].simul_info->results[num] = -1; */
    player_globals.parray[p].simul_info->boards[num] = player_globals.parray[p].game;
    player_globals.parray[p].simul_info->numBoards++;
    if (player_globals.parray[p].simul_info->numBoards > 1 &&
	player_globals.parray[p].simul_info->onBoard >= 0)
      player_goto_board(p, player_globals.parray[p].simul_info->onBoard);
    else
      player_globals.parray[p].simul_info->onBoard = 0;
    return COM_OK;
  }
  if (find_pend(-1, p, PEND_SIMUL) != NULL) {
    pprintf(p, "You cannot be the simul giver and request to join another simul.\nThat would just be too confusing for me and you.\n");
    return COM_OK;
  }
  if (player_globals.parray[p].simul_info != NULL) {
    if (player_globals.parray[p].simul_info->numBoards) {
      pprintf(p, "You cannot be the simul giver and request to join another simul.\nThat would just be too confusing for me and you.\n");
      return COM_OK;
    }
  }
  if (player_globals.parray[p].game >=0) {
    pprintf(p, "You are already playing a game.\n");
    return COM_OK;
  }
  if (!CheckPFlag(p1, PFLAG_SIMOPEN)) {
    pprintf_highlight(p, "%s", player_globals.parray[p1].name);
    pprintf(p, " is not open to receiving simul requests.\n");
    return COM_OK;
  }
  if ((player_globals.parray[p1].simul_info != NULL) && (player_globals.parray[p1].simul_info->numBoards >= MAX_SIMUL)) {
    pprintf_highlight(p, "%s", player_globals.parray[p1].name);
    pprintf(p, " is already playing the maximum of %d boards.\n", MAX_SIMUL);
    return COM_OK;
  }

/* loon: checking for some crazy situations we can't allow :) */

  if ((player_globals.parray[p1].simul_info != NULL) && (player_globals.parray[p1].game >=0) && (player_globals.parray[p1].simul_info->numBoards == 0)) {
    pprintf_highlight(p, "%s", player_globals.parray[p1].name);
    if (player_globals.parray[game_globals.garray[player_globals.parray[p1].game].white].simul_info->numBoards) {
      pprintf(p, " is playing in ");
      pprintf_highlight(p, "%s", player_globals.parray[player_globals.parray[p1].opponent].name);
      pprintf(p, "'s simul, and can't accept.\n");
    } else {
      pprintf(p, " can't begin a simul while playing a non-simul game.\n");
    }
    return COM_OK;
  }

  g = game_new();		/* Check if an adjourned untimed game */
  adjourned = ((game_read(g, p, p1) < 0) && (game_read(g, p1, p) < 0)) ? 0 : 1;
  if (adjourned) {
    if (!(game_globals.garray[g].type == TYPE_UNTIMED))
      adjourned = 0;
  }
  game_remove(g);

  pend = add_request(p, p1, PEND_SIMUL);

  if ((param[1].type == TYPE_WORD) && (param[2].type == TYPE_WORD)) {

    sprintf(fname, "%s/%s/%s", BOARD_DIR, param[1].val.word , param[2].val.word);
    if (!file_exists(fname)) {
      pprintf(p, "No such category/board: %s/%s\n", param[1].val.word , param[2].val.word);
      return COM_OK;
    }
    pend->category = strdup(param[1].val.word);
    pend->board_type = strdup(param[2].val.word);
  } else {
    pend->category = NULL;
    pend->board_type = NULL;
  }
 
  pprintf(p1, "\n");
  pprintf_highlight(p1, "%s", player_globals.parray[p].name);
  if (adjourned) {
    pprintf_prompt(p1, " requests to continue an adjourned simul game.\n");
    pprintf(p, "Request to resume simul sent. Adjourned game found.\n");
  } else {
    if (pend->category == NULL)
      pprintf_prompt(p1, " requests to join a simul match with you.\n");
    else
      pprintf_prompt(p1, " requests to join a %s %s simul match with you.\n",
                pend->category,pend->board_type);
    pprintf(p, "Simul match request sent.\n");
  }
  return COM_OK;
}

int com_goboard(int p, param_list param)
{
  int on, g, p1, gamenum;

  if (player_globals.parray[p].simul_info == NULL) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  if (!player_globals.parray[p].simul_info->numBoards) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  if (param[0].type == TYPE_WORD) {

    p1 = player_find_part_login(param[0].val.word);
    if (p1 < 0) {
      pprintf(p, "No user named \"%s\" is logged in.\n", param[0].val.word);
      return COM_OK;
    }
    if (p == p1) {
      pprintf(p, "You can't goboard yourself!\n");
      return COM_OK;
    }

    gamenum = player_globals.parray[p1].game;
    if (gamenum < 0) {
      pprintf (p,"%s is not playing a game.\n", player_globals.parray[p1].login);
      return COM_OK;
    }

  } else { 
    gamenum = param[0].val.integer - 1;
    if (gamenum < 0)
      gamenum = 0;
  }

  on = player_globals.parray[p].simul_info->onBoard;
  g = player_globals.parray[p].simul_info->boards[on];
  if (gamenum == g) {
    pprintf(p, "You are already at that board!\n");
    return COM_OK;
  }
  if (player_globals.parray[p].simul_info->numBoards > 1) {
    decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);
    if (player_goto_simulgame_bynum(p, gamenum) !=-1) {
      if (g >= 0) {
	pprintf(game_globals.garray[g].black, "\n");
	pprintf_highlight(game_globals.garray[g].black, "%s", player_globals.parray[p].name);
	pprintf_prompt(game_globals.garray[g].black, " has moved away from your board.\n");
      }
    } else
    pprintf(p, "You are not playing that game/person.\n");
  } else
    pprintf(p, "You are only playing one board!\n");
  return COM_OK;
}

int com_simnext(int p, param_list param)
{
  int on, g;

  if (player_globals.parray[p].simul_info == NULL) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  if (!player_globals.parray[p].simul_info->numBoards) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  if (player_globals.parray[p].simul_info->numBoards > 1) {
    decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);
    on = player_globals.parray[p].simul_info->onBoard;
    g = player_globals.parray[p].simul_info->boards[on];
    if (g >= 0) {
      pprintf(game_globals.garray[g].black, "\n");
      pprintf_highlight(game_globals.garray[g].black, "%s", player_globals.parray[p].name);
      pprintf_prompt(game_globals.garray[g].black, " is moving away from your board.\n");
      player_goto_next_board(p);
    }
  } else
    pprintf(p, "You are only playing one board!\n");
  return COM_OK;
}

int com_simprev(int p, param_list param)
{
  int on, g;

  if (player_globals.parray[p].simul_info == NULL) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  if (!player_globals.parray[p].simul_info->numBoards) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }
  if (player_globals.parray[p].simul_info->numBoards > 1) {
    decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);
    on = player_globals.parray[p].simul_info->onBoard;
    g = player_globals.parray[p].simul_info->boards[on];
    if (g >= 0) {
      pprintf(game_globals.garray[g].black, "\n");
      pprintf_highlight(game_globals.garray[g].black, "%s", player_globals.parray[p].name);
      pprintf_prompt(game_globals.garray[g].black, " is moving back to the previous board.\n");
    }
    player_goto_prev_board(p);
  } else
    pprintf(p, "You are only playing one board!\n");
  return COM_OK;
}

int com_simgames(int p, param_list param)
{
  int p1 = p;

  if (param[0].type == TYPE_WORD) {
    if ((p1 = player_find_part_login(param[0].val.word)) < 0) {
      pprintf(p, "No player named %s is logged in.\n", param[0].val.word);
      return COM_OK;
    }
  }
  if (p1 == p)
    pprintf(p, "You are playing %d simultaneous games.\n",
	    player_num_active_boards(p1));
  else
    pprintf(p, "%s is playing %d simultaneous games.\n", player_globals.parray[p1].name,
	    player_num_active_boards(p1));
  return COM_OK;
}

int com_simpass(int p, param_list param)
{
  int g, p1, on;

  if (!pIsPlaying(p))
    return COM_OK;

  g = player_globals.parray[p].game;
  p1 = game_globals.garray[g].white;

  if (player_globals.parray[p1].simul_info == NULL) {
    pprintf(p, "You are not participating in a simul.\n");
    return COM_OK;
  }

  if (!player_globals.parray[p1].simul_info->numBoards) {
    pprintf(p, "You are not participating in a simul.\n");
    return COM_OK;
  }
  if (p == p1) {
    pprintf(p, "You are the simul holder and cannot pass!\n");
    return COM_OK;
  }
  if (player_num_active_boards(p1) == 1) {
    pprintf(p, "This is the only game, so passing is futile.\n");
    return COM_OK;
  }
  on = player_globals.parray[p1].simul_info->onBoard;
  if (player_globals.parray[p1].simul_info->boards[on] != g) {
    pprintf(p, "You cannot pass until the simul holder arrives!\n");
    return COM_OK;
  }
  if (game_globals.garray[g].passes >= MAX_SIMPASS) {
    Bell (p);
    pprintf(p, "You have reached your maximum of %d pass(es).\n", MAX_SIMPASS);
    pprintf(p, "Please move IMMEDIATELY!\n");
    pprintf_highlight(p1, "%s", player_globals.parray[p].name);
    pprintf_prompt(p1, " tried to pass, but is out of passes.\n");
    return COM_OK;
  }
  decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);

  game_globals.garray[g].passes++;
  pprintf(p, "You have passed and have %d pass(es) left.\n",
	  (MAX_SIMPASS - game_globals.garray[g].passes));
  pprintf_highlight(p1, "%s", player_globals.parray[p].name);
  pprintf_prompt(p1, " has decided to pass and has %d pass(es) left.\n",
		 (MAX_SIMPASS - game_globals.garray[g].passes));
  player_goto_next_board(p1);
  return COM_OK;
}

int com_simabort(int p, param_list param)
{

  if (player_globals.parray[p].simul_info == NULL) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  if (!player_globals.parray[p].simul_info->numBoards) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }
  decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);
  game_ended(player_globals.parray[p].simul_info->boards[player_globals.parray[p].simul_info->onBoard],
	     WHITE, END_ABORT);
  return COM_OK;
}

int com_simallabort(int p, param_list param)
{
  int i;

  if (player_globals.parray[p].simul_info == NULL) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  if (!player_globals.parray[p].simul_info->numBoards) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);
  for (i = 0; i < player_globals.parray[p].simul_info->numBoards; i++)
    if (player_globals.parray[p].simul_info->boards[i] >= 0)
      game_ended(player_globals.parray[p].simul_info->boards[i],
		 WHITE, END_ABORT);

  return COM_OK;
}

int com_simadjourn(int p, param_list param)
{

  if (player_globals.parray[p].simul_info == NULL) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  if (!player_globals.parray[p].simul_info->numBoards) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }
  decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);
  game_ended(player_globals.parray[p].simul_info->boards[player_globals.parray[p].simul_info->onBoard],
	     WHITE, END_ADJOURN);
  return COM_OK;
}

int com_simalladjourn(int p, param_list param)
{
  int i;

  if (player_globals.parray[p].simul_info == NULL) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }

  if (!player_globals.parray[p].simul_info->numBoards) {
    pprintf(p, "You are not giving a simul.\n");
    return COM_OK;
  }
  decline_withdraw_offers(p, -1, -PEND_SIMUL,DO_DECLINE);
  for (i = 0; i < player_globals.parray[p].simul_info->numBoards; i++)
    if (player_globals.parray[p].simul_info->boards[i] >= 0)
      game_ended(player_globals.parray[p].simul_info->boards[i],
		 WHITE, END_ADJOURN);

  return COM_OK;
}

int com_moretime(int p, param_list param)
{
  int g, increment;

  if ((player_globals.parray[p].game >=0) &&((game_globals.garray[player_globals.parray[p].game].status == GAME_EXAMINE) ||
        (game_globals.garray[player_globals.parray[p].game].status == GAME_SETUP))) {
    pprintf(p, "You cannot use moretime in an examined game.\n");
    return COM_OK;
  }
  increment = param[0].val.integer;
  if (increment <= 0) {
    pprintf(p, "Moretime requires an integer value greater than zero.\n");
    return COM_OK;
  }
  if (!pIsPlaying(p))
    return COM_OK;
 
  if (increment > 600) {
    pprintf(p, "Moretime has a maximum limit of 600 seconds.\n");
    increment = 600;
  }
  g = player_globals.parray[p].game;
  if (game_globals.garray[g].white == p) {
    game_globals.garray[g].bTime += increment * 10;
    game_globals.garray[g].bRealTime += increment * 10 * 100;
    pprintf(p, "%d seconds were added to your opponents clock\n",
	    increment);
    pprintf_prompt(player_globals.parray[p].opponent,
		   "\nYour opponent has added %d seconds to your clock.\n",
		   increment);
  }
  if (game_globals.garray[g].black == p) {
    game_globals.garray[g].wTime += increment * 10;;
    game_globals.garray[g].wRealTime += increment * 10 * 100;
    pprintf(p, "%d seconds were added to your opponents clock\n",
	    increment);
    pprintf_prompt(player_globals.parray[p].opponent,
		   "\nYour opponent has added %d seconds to your clock.\n",
		   increment);
  }
  return COM_OK;
}

